#include <stdio.h>
#include <stdlib.h>

#define MAXN 1000000
#define MAXM 10000000

struct vertex;

struct edge
{
  struct vertex *to;
  struct edge *opposite, *prev, *next;
};

struct vertex
{
  int mark;
  struct edge nbrs_head;
  struct vertex *next_deg1;
  struct vertex *next_dfs;
  struct vertex *prev_v, *next_v;
} g[MAXN];

static void
add_after (struct edge *af, struct edge *co)
{
  co->prev = af;
  co->next = af->next;
  co->prev->next = co;
  co->next->prev = co;
}

static int
degle1 (struct vertex *v)
{
  return v->nbrs_head.next->to == NULL || v->nbrs_head.next->next->to == NULL;
}

static int
cutvertex (struct vertex *v)
{
  static int mark_time = 0;
  struct vertex *stack;
  struct edge *e;

  if (degle1 (v))
    return 0;

  mark_time++;
  v->mark = mark_time;
  stack = v->nbrs_head.next->to;
  stack->next_dfs = NULL;

  while (stack)
    {
      struct vertex *a = stack;
      stack = stack->next_dfs;

      for (e = a->nbrs_head.next; e->to; e = e->next)
	{
	  if (e->to->mark == mark_time)
	    continue;

	  e->to->mark = mark_time;
	  e->to->next_dfs = stack;
	  stack = e->to;
	}
    }

  for (e = v->nbrs_head.next; e->to; e = e->next)
    if (e->to->mark != mark_time)
      return 1;

  return 0;
}

static void
remove_halfedge (struct edge *e)
{
  struct edge *p, *n;

  p = e->prev;
  n = e->next;
  p->next = n;
  n->prev = p;
  free (e);
}

static void
remove_edge (struct edge *e, struct vertex **deg1)
{
  struct edge *ope = e->opposite;
  struct vertex *to = e->to;
  int td1 = !degle1 (to);

  remove_halfedge (e);
  remove_halfedge (ope);

  if (td1 && degle1 (to))
    {
      to->next_deg1 = *deg1;
      *deg1 = to;
    }
}

int main(void)
{
  int n, m;
  int u, v, i;
  struct edge *eu, *ev;
  struct vertex *deg1 = NULL;
  struct vertex vs_head;

  scanf("%d%d", &n, &m);
  for (i = 0; i < n; i++)
    {
      g[i].nbrs_head.to = NULL;
      g[i].nbrs_head.opposite = NULL;
      g[i].nbrs_head.prev = g[i].nbrs_head.next = &g[i].nbrs_head;
      g[i].prev_v = i > 0 ? &g[i - 1] : &vs_head;
      g[i].next_v = i < n - 1 ? &g[i + 1] : &vs_head;
    }
  vs_head.next_v = &g[0];
  vs_head.prev_v = &g[n-1];
  for (i = 0; i < m; i++)
    {
      scanf("%d%d", &u, &v);
      u--; v--;
      eu = malloc (sizeof (struct edge));
      ev = malloc (sizeof (struct edge));
      eu->to = g + v;
      eu->opposite = ev;
      ev->to = g + u;
      ev->opposite = eu;
      add_after (&g[u].nbrs_head, eu);
      add_after (&g[v].nbrs_head, ev);
    }
  for (i = 0; i < n; i++)
    if (degle1 (&g[i]))
      {
	g[i].next_deg1 = deg1;
	deg1 = &g[i];
      }


  for (i = 0; i < n; i++)
    {
      struct vertex *rem, *pv, *nx;

      if (deg1)
	{
	  rem = deg1;
	  deg1 = deg1->next_deg1;
	}
      else
	{
	  for (rem = vs_head.next_v; cutvertex (rem); rem = rem->next_v)
	    continue;
	}

      printf ("%ld\n", (rem - g) + 1);

      while (rem->nbrs_head.next->to)
	remove_edge (rem->nbrs_head.next, &deg1);

      pv = rem->prev_v;
      nx = rem->next_v;
      pv->next_v = nx;
      nx->prev_v = pv;
    }

  return 0;
}
